@@ -51,12 +51,12 @@
 
 #include "../modules/default@2025-06-18.h"
 #include "../modules/ietf-datastores@2018-02-14.h"
-#include "../modules/ietf-inet-types@2013-07-15.h"
+#include "../modules/ietf-inet-types@2025-12-22.h"
 #include "../modules/ietf-yang-library@2019-01-04.h"
 #include "../modules/ietf-yang-metadata@2016-08-05.h"
 #include "../modules/ietf-yang-schema-mount@2019-01-14.h"
 #include "../modules/ietf-yang-structure-ext@2020-06-17.h"
-#include "../modules/ietf-yang-types@2013-07-15.h"
+#include "../modules/ietf-yang-types@2025-12-22.h"
 #include "../modules/yang@2025-01-29.h"
 #define IETF_YANG_LIB_REV "2019-01-04"
 
@@ -67,16 +67,16 @@ static struct internal_modules_s {
     ly_bool implemented;
     LYS_INFORMAT format;
 } internal_modules[] = {
-    {"ietf-yang-metadata", "2016-08-05", (const char *)ietf_yang_metadata_2016_08_05_yang, 1, LYS_IN_YANG},
-    {"yang", "2025-01-29", (const char *)yang_2025_01_29_yang, 1, LYS_IN_YANG},
-    {"default", "2025-06-18", (const char *)default_2025_06_18_yang, 1, LYS_IN_YANG},
-    {"ietf-inet-types", "2013-07-15", (const char *)ietf_inet_types_2013_07_15_yang, 0, LYS_IN_YANG},
-    {"ietf-yang-types", "2013-07-15", (const char *)ietf_yang_types_2013_07_15_yang, 0, LYS_IN_YANG},
-    {"ietf-yang-schema-mount", "2019-01-14", (const char *)ietf_yang_schema_mount_2019_01_14_yang, 1, LYS_IN_YANG},
-    {"ietf-yang-structure-ext", "2020-06-17", (const char *)ietf_yang_structure_ext_2020_06_17_yang, 0, LYS_IN_YANG},
+    {"ietf-inet-types", "2025-12-22", ietf_inet_types_2025_12_22_yang, 0, LYS_IN_YANG},
+    {"ietf-yang-types", "2025-12-22", ietf_yang_types_2025_12_22_yang, 0, LYS_IN_YANG},
+    {"ietf-yang-metadata", "2016-08-05", ietf_yang_metadata_2016_08_05_yang, 1, LYS_IN_YANG},
+    {"yang", "2025-01-29", yang_2025_01_29_yang, 1, LYS_IN_YANG},
+    {"default", "2025-06-18", default_2025_06_18_yang, 1, LYS_IN_YANG},
+    {"ietf-yang-schema-mount", "2019-01-14", ietf_yang_schema_mount_2019_01_14_yang, 1, LYS_IN_YANG},
+    {"ietf-yang-structure-ext", "2020-06-17", ietf_yang_structure_ext_2020_06_17_yang, 0, LYS_IN_YANG},
     /* ietf-datastores and ietf-yang-library must be right here at the end of the list! */
-    {"ietf-datastores", "2018-02-14", (const char *)ietf_datastores_2018_02_14_yang, 1, LYS_IN_YANG},
-    {"ietf-yang-library", IETF_YANG_LIB_REV, (const char *)ietf_yang_library_2019_01_04_yang, 1, LYS_IN_YANG}
+    {"ietf-datastores", "2018-02-14", ietf_datastores_2018_02_14_yang, 1, LYS_IN_YANG},
+    {"ietf-yang-library", IETF_YANG_LIB_REV, ietf_yang_library_2019_01_04_yang, 1, LYS_IN_YANG}
 };
 
 #define LY_INTERNAL_MODS_COUNT sizeof(internal_modules) / sizeof(struct internal_modules_s)
@@ -178,28 +178,39 @@ ly_ctx_get_searchdirs(const struct ly_ctx *ctx)
 }
 
 LIBYANG_API_DEF LY_ERR
-ly_ctx_unset_searchdir(struct ly_ctx *ctx, const char *value)
+ly_ctx_unset_searchdir(struct ly_ctx *ctx, const char *search_dir)
 {
+    LY_ERR rc = LY_SUCCESS;
+    uint32_t index;
+    char *search_dir_real = NULL;
+
     LY_CHECK_ARG_RET(ctx, ctx, !(ctx->opts & LY_CTX_INT_IMMUTABLE), LY_EINVAL);
 
     if (!ctx->search_paths.count) {
         return LY_SUCCESS;
     }
 
-    if (value) {
-        /* remove specific search directory */
-        uint32_t index;
+    if (search_dir) {
+        search_dir_real = realpath(search_dir, NULL);
+        if (!search_dir_real) {
+            LOGERR(ctx, LY_EINVAL, "Unable to normalize search directory \"%s\" (%s).", search_dir, strerror(errno));
+            rc = LY_EINVAL;
+            goto cleanup;
+        }
 
+        /* remove specific search directory */
         for (index = 0; index < ctx->search_paths.count; ++index) {
-            if (!strcmp(value, ctx->search_paths.objs[index])) {
+            if (!strcmp(search_dir_real, ctx->search_paths.objs[index])) {
                 break;
             }
         }
         if (index == ctx->search_paths.count) {
-            LOGARG(ctx, value);
-            return LY_EINVAL;
+            LOGARG(ctx, search_dir_real);
+            rc = LY_EINVAL;
+            goto cleanup;
         } else {
-            return ly_set_rm_index(&ctx->search_paths, index, free);
+            rc = ly_set_rm_index(&ctx->search_paths, index, free);
+            goto cleanup;
         }
     } else {
         /* remove them all */
@@ -207,7 +218,9 @@ ly_ctx_unset_searchdir(struct ly_ctx *ctx, const char *value)
         memset(&ctx->search_paths, 0, sizeof ctx->search_paths);
     }
 
-    return LY_SUCCESS;
+cleanup:
+    free(search_dir_real);
+    return rc;
 }
 
 LIBYANG_API_DEF LY_ERR
@@ -751,77 +764,38 @@ ly_ctx_get_modules_hash(const struct ly_ctx *ctx)
     return ctx->mod_hash;
 }
 
-/**
- * @brief qsort() alphabetical compare of modules.
- */
-static int
-ly_ctx_mod_cmp_cb(const void *ptr1, const void *ptr2)
-{
-    const struct lys_module *mod1, *mod2;
-    int r;
-
-    mod1 = *(struct lys_module **)ptr1;
-    mod2 = *(struct lys_module **)ptr2;
-
-    /* module name */
-    r = strcmp(mod1->name, mod2->name);
-    if (!r) {
-        /* revision */
-        if (mod1->revision && !mod2->revision) {
-            r = 1;
-        } else if (!mod1->revision && mod2->revision) {
-            r = -1;
-        } else if (mod1->revision && mod2->revision) {
-            r = strcmp(mod1->revision, mod2->revision);
-        }
-    }
-
-    return r;
-}
-
 void
 ly_ctx_new_change(struct ly_ctx *ctx)
 {
-    uint32_t i, hash = 0;
-    const struct lys_module **mods = NULL;
+    const struct lys_module *mod;
+    uint32_t i = ly_ctx_internal_modules_count(ctx), hash = 0;
     LY_ARRAY_COUNT_TYPE u;
 
     /* change counter */
     ctx->change_count++;
 
-    /* collect modules into an array */
-    mods = malloc(ctx->modules.count * sizeof *mods);
-    LY_CHECK_ERR_GOTO(!mods, LOGMEM(ctx), cleanup);
-    memcpy(mods, ctx->modules.objs, ctx->modules.count * sizeof *mods);
-
-    /* sort modules alphabetically */
-    qsort(mods, ctx->modules.count, sizeof *mods, ly_ctx_mod_cmp_cb);
-
-    /* calculate module hash */
-    for (i = 0; i < ctx->modules.count; ++i) {
+    /* module hash */
+    while ((mod = ly_ctx_get_module_iter(ctx, &i))) {
         /* name */
-        hash = lyht_hash_multi(hash, mods[i]->name, strlen(mods[i]->name));
+        hash = lyht_hash_multi(hash, mod->name, strlen(mod->name));
 
         /* revision */
-        if (mods[i]->revision) {
-            hash = lyht_hash_multi(hash, mods[i]->revision, strlen(mods[i]->revision));
+        if (mod->revision) {
+            hash = lyht_hash_multi(hash, mod->revision, strlen(mod->revision));
         }
 
         /* enabled features */
-        if (mods[i]->implemented) {
-            LY_ARRAY_FOR(mods[i]->compiled->features, u) {
-                hash = lyht_hash_multi(hash, mods[i]->compiled->features[u], strlen(mods[i]->compiled->features[u]));
+        if (mod->implemented) {
+            LY_ARRAY_FOR(mod->compiled->features, u) {
+                hash = lyht_hash_multi(hash, mod->compiled->features[u], strlen(mod->compiled->features[u]));
             }
         }
 
         /* imported/implemented */
-        hash = lyht_hash_multi(hash, (char *)&mods[i]->implemented, sizeof mods[i]->implemented);
+        hash = lyht_hash_multi(hash, (char *)&mod->implemented, sizeof mod->implemented);
     }
 
     ctx->mod_hash = lyht_hash_multi(hash, NULL, 0);
-
-cleanup:
-    free(mods);
 }
 
 LIBYANG_API_DEF ly_module_imp_clb
@@ -1535,7 +1509,8 @@ ly_ctx_destroy(struct ly_ctx *ctx)
 
     if (ctx->opts & LY_CTX_INT_IMMUTABLE) {
         if (!ctx->parent_ctx) {
-            /* ctx data */
+            /* cached patterns and ctx data */
+            ly_ctx_pattern_ht_erase(ctx);
             ly_ctx_data_del(ctx);
         }
         lyplg_clean();
@@ -1555,11 +1530,15 @@ ly_ctx_destroy(struct ly_ctx *ctx)
             mod->compiled = NULL;
         }
 
-        /* even compiled ext definitions can have ext instances, free those, too */
+        /* even compiled ext definitions and itentities can have ext instances, free those, too */
         LY_ARRAY_FOR(mod->extensions, u) {
             FREE_ARRAY(ctx, mod->extensions[u].exts, lysc_ext_instance_free);
             mod->extensions[u].exts = NULL;
         }
+        LY_ARRAY_FOR(mod->identities, u) {
+            FREE_ARRAY(ctx, mod->identities[u].exts, lysc_ext_instance_free);
+            mod->identities[u].exts = NULL;
+        }
     }
 
     /* free the modules */
