@@ -86,6 +86,13 @@ lyb_read(void *buf, uint64_t count_bits, struct lylyb_parse_ctx *lybctx)
         return;
     }
 
+    if (!lybctx->shrink) {
+        /* just read full bytes */
+        count_bytes = LYPLG_BITS2BYTES(count_bits);
+        ly_in_read(lybctx->in, buf, count_bytes);
+        return;
+    }
+
     if (lybctx->buf_bits) {
         /* will spent buffered bits */
         count_buf_bits = (count_bits > lybctx->buf_bits) ? lybctx->buf_bits : count_bits;
@@ -138,44 +145,6 @@ lyb_read(void *buf, uint64_t count_bits, struct lylyb_parse_ctx *lybctx)
     }
 }
 
-/**
- * @brief Peek data from the input.
- *
- * @param[in,out] buf Destination buffer, @p count_bits rightmost bits are written to, rest are shifted left.
- * @param[in] count_bits Number of bits to peek.
- * @param[in] lybctx LYB context.
- */
-static void
-lyb_peek(void *buf, uint64_t count_bits, struct lylyb_parse_ctx *lybctx)
-{
-    uint8_t count_buf_bits = 0, peek;
-
-    assert(lybctx && (count_bits < 9));
-
-    if (!count_bits) {
-        return;
-    }
-
-    if (lybctx->buf_bits) {
-        /* peek the buffered bits */
-        count_buf_bits = (lybctx->buf_bits > count_bits) ? count_bits : lybctx->buf_bits;
-
-        /* keep the original leftmost bits and write new peeked rightmost bits */
-        ((uint8_t *)buf)[0] = (((uint8_t *)buf)[0] << count_bits) | (lybctx->buf & lyb_right_bit_mask(count_buf_bits));
-        count_bits -= count_buf_bits;
-    }
-
-    if (!count_bits) {
-        return;
-    }
-
-    /* need to peek input */
-    ly_in_peek(lybctx->in, &peek);
-
-    /* prepend the newly peeked bits in front of the previously peeked bits */
-    ((uint8_t *)buf)[0] |= (peek & lyb_right_bit_mask(count_bits)) << count_buf_bits;
-}
-
 /**
  * @brief Read a count.
  *
@@ -185,9 +154,20 @@ lyb_peek(void *buf, uint64_t count_bits, struct lylyb_parse_ctx *lybctx)
 static void
 lyb_read_count(uint32_t *count, struct lylyb_parse_ctx *lybctx)
 {
-    uint8_t prefix = 0, pref_len = 0;
+    uint8_t prefix = 0, pref_len = 0, byte_len = 0;
+    uint16_t count16 = 0;
 
-    /* read all prefix bits */
+    /* --- no shrink mode --- */
+    if (!lybctx->shrink) {
+        /* always use 2 bytes for the count */
+        byte_len = 2;
+        lyb_read(&count16, byte_len * 8, lybctx);
+        *count = le16toh(count16);
+        return;
+    }
+
+    /* --- shrink mode ---
+     * read all prefix bits */
     do {
         ++pref_len;
         prefix <<= 1;
@@ -238,9 +218,19 @@ lyb_read_count(uint32_t *count, struct lylyb_parse_ctx *lybctx)
 static void
 lyb_read_size(uint32_t *size, struct lylyb_parse_ctx *lybctx)
 {
-    uint8_t prefix = 0, pref_len = 0;
+    uint8_t prefix = 0, pref_len = 0, byte_len = 0;
 
-    /* read all prefix bits */
+    /* --- no shrink mode --- */
+    if (!lybctx->shrink) {
+        /* always use 4 bytes for the size */
+        byte_len = 4;
+        lyb_read(size, byte_len * 8, lybctx);
+        *size = le32toh(*size);
+        return;
+    }
+
+    /* --- shrink mode ---
+     * read all prefix bits */
     do {
         ++pref_len;
         prefix <<= 1;
@@ -360,35 +350,179 @@ lyb_read_value(const struct lysc_type *type, uint8_t **val, uint32_t *val_size_b
 }
 
 /**
- * @brief Parse YANG module info stored separately as module name and its revision.
+ * @brief Compare module revision from LYB data with the module in the parser context.
  *
  * @param[in] lybctx LYB context.
- * @param[out] mod_name Module name, if any.
- * @param[out] mod_rev Module revision, "" if none.
+ * @param[in] mod_name Module name from LYB data.
+ * @param[in] mod_rev Module revision from LYB data.
+ * @param[in] mod Module from parser context.
  * @return LY_ERR value.
  */
 static LY_ERR
-lyb_parse_module(struct lylyb_parse_ctx *lybctx, char **mod_name, char mod_rev[])
+lyb_check_mod_revision(struct lyd_lyb_ctx *lybctx, const char *mod_name, const char *mod_rev,
+        const struct lys_module *mod)
 {
-    uint16_t rev;
+    ly_bool rev;
 
-    *mod_name = NULL;
-    mod_rev[0] = '\0';
-
-    /* module name */
-    LY_CHECK_RET(lyb_read_string(mod_name, lybctx));
-    if (!(*mod_name)[0]) {
+    if (lybctx->parse_opts & LYD_PARSE_LYB_SKIP_MODULE_CHECK) {
+        /* skip revision check */
         return LY_SUCCESS;
     }
 
+    rev = (mod_rev[0] != '\0');
+    if ((rev && !mod->revision) || (!rev && mod->revision) || (rev && strcmp(mod->revision, mod_rev))) {
+        LOGERR(lybctx->parse_ctx->ctx, LY_EDENIED,
+                "Invalid context for LYB data parsing, module \"%s\" revision mismatch (printer: %s, parser: %s).",
+                mod_name,
+                rev ? mod_rev : "<none>",
+                mod->revision ? mod->revision : "<none>");
+        return LY_EDENIED;
+    }
+
+    return LY_SUCCESS;
+}
+
+/**
+ * @brief Compare enabled module features from LYB data with the module in the parser context.
+ *
+ * @param[in] lybctx LYB context.
+ * @param[in] mod_name Module name from LYB data.
+ * @param[in] mod Module from parser context.
+ * @return LY_ERR value.
+ */
+static LY_ERR
+lyb_check_mod_features(struct lyd_lyb_ctx *lybctx, const char *mod_name, const struct lys_module *mod)
+{
+    LY_ERR rc = LY_SUCCESS;
+    struct lylyb_parse_ctx *pctx = lybctx->parse_ctx;
+    char *feature_name = NULL, **features = NULL, *en_feats_printer = NULL, *en_feats_parser = NULL;
+    uint32_t i, feature_count = 0, ctx_feature_count, len;
+
+    /* read feature count */
+    lyb_read_count(&feature_count, pctx);
+
+    if (feature_count) {
+        /* create an array for the features */
+        features = calloc(feature_count, sizeof *features);
+        LY_CHECK_ERR_RET(!features, LOGMEM(pctx->ctx), LY_EMEM);
+    }
+
+    /* read all features from LYB data */
+    for (i = 0; i < feature_count; i++) {
+        LY_CHECK_GOTO(rc = lyb_read_string(&feature_name, pctx), cleanup);
+        features[i] = feature_name;
+        feature_name = NULL;
+    }
+
+    if (lybctx->parse_opts & LYD_PARSE_LYB_SKIP_MODULE_CHECK) {
+        /* skip feature check */
+        goto cleanup;
+    }
+
+    /* we assume the order of features is the same (since the revision is), so we can just compare them in order */
+    if (feature_count == LY_ARRAY_COUNT(mod->compiled->features)) {
+        for (i = 0; i < feature_count; i++) {
+            if (strcmp(mod->compiled->features[i], features[i])) {
+                /* mismatch */
+                break;
+            }
+        }
+
+        if (i == feature_count) {
+            /* success */
+            goto cleanup;
+        }
+    }
+
+    /* feature mismatch, prepare error message, unpack enabled features for printer */
+    len = 0;
+    for (i = 0; i < feature_count; i++) {
+        len += strlen(features[i]) + 2;
+    }
+    en_feats_printer = calloc(1, len + 1);
+    LY_CHECK_ERR_GOTO(!en_feats_printer, LOGMEM(pctx->ctx), cleanup);
+    for (i = 0; i < feature_count; i++) {
+        strcat(en_feats_printer, features[i]);
+        if (i + 1 < feature_count) {
+            strcat(en_feats_printer, ", ");
+        }
+    }
+
+    /* unpack enabled features for parser */
+    len = 0;
+    ctx_feature_count = LY_ARRAY_COUNT(mod->compiled->features);
+    for (i = 0; i < ctx_feature_count; i++) {
+        len += strlen(mod->compiled->features[i]) + 2;
+    }
+    en_feats_parser = calloc(1, len + 1);
+    LY_CHECK_ERR_GOTO(!en_feats_parser, LOGMEM(pctx->ctx), cleanup);
+    for (i = 0; i < ctx_feature_count; i++) {
+        strcat(en_feats_parser, mod->compiled->features[i]);
+        if (i + 1 < ctx_feature_count) {
+            strcat(en_feats_parser, ", ");
+        }
+    }
+
+    LOGERR(pctx->ctx, LY_EDENIED,
+            "Invalid context for LYB data parsing, module \"%s\" enabled feature mismatch (printer: {%s}, parser: {%s}).",
+            mod_name, en_feats_printer, en_feats_parser);
+    rc = LY_EDENIED;
+
+cleanup:
+    free(en_feats_printer);
+    free(en_feats_parser);
+    for (i = 0; i < feature_count; i++) {
+        free(features[i]);
+    }
+    free(features);
+    return rc;
+}
+
+/**
+ * @brief Parse YANG module info as name, revision, and features.
+ *
+ * @param[in] lybctx LYB context.
+ * @param[out] mod Parsed module.
+ * @return LY_ERR value.
+ */
+static LY_ERR
+lyb_parse_module(struct lyd_lyb_ctx *lybctx, const struct lys_module **mod)
+{
+    LY_ERR rc = LY_SUCCESS;
+    struct lylyb_parse_ctx *pctx = lybctx->parse_ctx;
+    char *mod_name = NULL, mod_rev[LY_REV_SIZE];
+    uint16_t rev;
+
+    mod_rev[0] = '\0';
+
+    /* module name */
+    LY_CHECK_RET(lyb_read_string(&mod_name, pctx));
+    assert(mod_name[0]);
+
+    /* find the module in the context */
+    *mod = ly_ctx_get_module_implemented(pctx->ctx, mod_name);
+    if (!*mod) {
+        LOGERR(pctx->ctx, LY_EDENIED, "Invalid context for LYB data parsing, module \"%s\" not found.", mod_name);
+        rc = LY_EDENIED;
+        goto cleanup;
+    }
+
     /* module revision */
-    lyb_read(&rev, 2 * 8, lybctx);
+    lyb_read(&rev, 2 * 8, pctx);
     if (rev) {
         sprintf(mod_rev, "%04u-%02u-%02u", ((rev & LYB_REV_YEAR_MASK) >> LYB_REV_YEAR_SHIFT) + LYB_REV_YEAR_OFFSET,
                 (rev & LYB_REV_MONTH_MASK) >> LYB_REV_MONTH_SHIFT, rev & LYB_REV_DAY_MASK);
     }
 
-    return LY_SUCCESS;
+    /* revision check */
+    LY_CHECK_GOTO(rc = lyb_check_mod_revision(lybctx, mod_name, mod_rev, *mod), cleanup);
+
+    /* features and feature check */
+    LY_CHECK_GOTO(rc = lyb_check_mod_features(lybctx, mod_name, *mod), cleanup);
+
+cleanup:
+    free(mod_name);
+    return rc;
 }
 
 /**
@@ -399,20 +533,28 @@ lyb_parse_module(struct lylyb_parse_ctx *lybctx, char **mod_name, char mod_rev[]
  * @return LY_ERR value.
  */
 static LY_ERR
-lyb_parse_module_idx(struct lylyb_parse_ctx *lybctx, const struct lys_module **mod)
+lyb_parse_module_idx(struct lyd_lyb_ctx *lybctx, const struct lys_module **mod)
 {
     LY_ERR rc = LY_SUCCESS;
+    struct lylyb_parse_ctx *pctx = lybctx->parse_ctx;
     const struct lys_module *m = NULL;
     uint32_t idx = 0;
 
     /* read module index */
-    lyb_read_count(&idx, lybctx);
+    lyb_read_count(&idx, pctx);
+
+    /* check the read index */
+    if (idx >= pctx->ctx->modules.count) {
+        LOGERR(pctx->ctx, LY_EINT, "Invalid context for LYB data parsing, module with index %" PRIu32 " not found.",
+                idx);
+        rc = LY_EINT;
+        goto cleanup;
+    }
 
     /* get the module */
-    assert(idx < lybctx->ctx->modules.count);
-    m = lybctx->ctx->modules.objs[idx];
+    m = pctx->ctx->modules.objs[idx];
     if (!m->implemented) {
-        LOGERR(lybctx->ctx, LY_EINT, "Invalid context for LYB data parsing, module \"%s%s%s\" not implemented.",
+        LOGERR(pctx->ctx, LY_EINT, "Invalid context for LYB data parsing, module \"%s%s%s\" not implemented.",
                 m->name, m->revision ? "@" : "", m->revision ? m->revision : "");
         rc = LY_EINT;
         goto cleanup;
@@ -428,11 +570,15 @@ cleanup:
  *
  * @param[in] lybctx LYB context.
  * @param[in] sparent Schema parent node of the metadata.
+ * @param[in] metadata_count Number of metadata stored.
+ * If UINT32_MAX, the count has not been read yet and should be read here.
+ * Can't use 0, because 0 is a valid count. We can use UINT32_MAX, because the maximum
+ * possible count is limited by ::LYB_METADATA_END_COUNT.
  * @param[out] meta Parsed metadata.
  * @return LY_ERR value.
  */
 static LY_ERR
-lyb_parse_metadata(struct lyd_lyb_ctx *lybctx, const struct lysc_node *sparent, struct lyd_meta **meta)
+lyb_parse_metadata(struct lyd_lyb_ctx *lybctx, const struct lysc_node *sparent, uint32_t metadata_count, struct lyd_meta **meta)
 {
     LY_ERR rc = LY_SUCCESS;
     ly_bool dynamic;
@@ -443,13 +589,21 @@ lyb_parse_metadata(struct lyd_lyb_ctx *lybctx, const struct lysc_node *sparent,
     struct lysc_ext_instance *ant;
     const struct lysc_type *ant_type;
 
-    /* read number of metadata stored */
-    lyb_read_count(&count, lybctx->parse_ctx);
+    if (metadata_count == UINT32_MAX) {
+        /* reserved value meaning metadata count not read yet, read it now */
+        lyb_read_count(&count, lybctx->parse_ctx);
+    } else {
+        /* use provided count */
+        count = metadata_count;
+    }
 
     for (i = 0; i < count; ++i) {
         /* find module */
-        rc = lyb_parse_module_idx(lybctx->parse_ctx, &mod);
-        LY_CHECK_GOTO(rc, cleanup);
+        if (lybctx->parse_ctx->shrink) {
+            LY_CHECK_GOTO(rc = lyb_parse_module_idx(lybctx, &mod), cleanup);
+        } else {
+            LY_CHECK_GOTO(rc = lyb_parse_module(lybctx, &mod), cleanup);
+        }
 
         /* meta name */
         rc = lyb_read_string(&meta_name, lybctx->parse_ctx);
@@ -949,19 +1103,23 @@ lyb_finish_node(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, uint32_t fl
  *
  * @param[in] lybctx LYB context.
  * @param[in] sparent Schema parent node of the metadata.
+ * @param[in] metadata_count Number of metadata already read.
  * @param[out] flags Parsed node flags.
  * @param[out] meta Parsed metadata of the node.
  * @return LY_ERR value.
  */
 static LY_ERR
-lyb_parse_node_header(struct lyd_lyb_ctx *lybctx, const struct lysc_node *sparent, uint32_t *flags, struct lyd_meta **meta)
+lyb_parse_node_header(struct lyd_lyb_ctx *lybctx, const struct lysc_node *sparent, uint32_t metadata_count, uint32_t *flags, struct lyd_meta **meta)
 {
     /* create and read metadata */
-    LY_CHECK_RET(lyb_parse_metadata(lybctx, sparent, meta));
+    LY_CHECK_RET(lyb_parse_metadata(lybctx, sparent, metadata_count, meta));
 
     if (!lybctx->parse_ctx->shrink) {
         /* read flags, fixed bits */
         lyb_read(flags, LYB_DATA_NODE_FLAG_BITS, lybctx->parse_ctx);
+
+        /* correct byte order */
+        *flags = le32toh(*flags);
     } else {
         *flags = LYD_NEW;
     }
@@ -1137,7 +1295,7 @@ lyb_parse_node_any(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const st
     const struct ly_ctx *ctx = lybctx->parse_ctx->ctx;
 
     /* read necessary basic data */
-    rc = lyb_parse_node_header(lybctx, snode, &flags, &meta);
+    rc = lyb_parse_node_header(lybctx, snode, UINT32_MAX, &flags, &meta);
     LY_CHECK_GOTO(rc, error);
 
     /* parse value type */
@@ -1218,7 +1376,7 @@ lyb_parse_node_inner(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const
     uint32_t flags = 0;
 
     /* read necessary basic data */
-    rc = lyb_parse_node_header(lybctx, snode, &flags, &meta);
+    rc = lyb_parse_node_header(lybctx, snode, UINT32_MAX, &flags, &meta);
     LY_CHECK_GOTO(rc, error);
 
     /* create node */
@@ -1262,13 +1420,14 @@ error:
  * @param[in] lybctx LYB context.
  * @param[in] parent Data parent of the sibling.
  * @param[in] snode Schema of the node to be parsed.
+ * @param[in] metadata_count Number of metadata already read for the node.
  * @param[in,out] first_p First top-level sibling.
  * @param[out] parsed Set of all successfully parsed nodes.
  * @return LY_ERR value.
  */
 static LY_ERR
 lyb_parse_node_leaf(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const struct lysc_node *snode,
-        struct lyd_node **first_p, struct ly_set *parsed)
+        uint32_t metadata_count, struct lyd_node **first_p, struct ly_set *parsed)
 {
     LY_ERR ret;
     struct lyd_node *node = NULL;
@@ -1276,7 +1435,7 @@ lyb_parse_node_leaf(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const s
     uint32_t flags = 0;
 
     /* read necessary basic data */
-    ret = lyb_parse_node_header(lybctx, snode, &flags, &meta);
+    ret = lyb_parse_node_header(lybctx, snode, metadata_count, &flags, &meta);
     LY_CHECK_GOTO(ret, error);
 
     /* read value of term node and create it */
@@ -1311,20 +1470,19 @@ static LY_ERR
 lyb_parse_node_leaflist(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const struct lysc_node *snode,
         struct lyd_node **first_p, struct ly_set *parsed)
 {
-    uint8_t peek;
+    uint32_t metadata_count;
 
     while (1) {
-        /* peek for the end instance flag (special metadata count) */
-        peek = 0;
-        lyb_peek(&peek, LYB_METADATA_END_BITS, lybctx->parse_ctx);
-        if (peek == LYB_METADATA_END) {
-            /* all the instances parsed, read the end flag */
-            lyb_read(&peek, LYB_METADATA_END_BITS, lybctx->parse_ctx);
+        /* read metadata count to check for end of instances */
+        metadata_count = 0;
+        lyb_read_count(&metadata_count, lybctx->parse_ctx);
+        if (metadata_count == LYB_METADATA_END_COUNT) {
+            /* all the instances parsed */
             break;
         }
 
         /* next instance */
-        LY_CHECK_RET(lyb_parse_node_leaf(lybctx, parent, snode, first_p, parsed));
+        LY_CHECK_RET(lyb_parse_node_leaf(lybctx, parent, snode, metadata_count, first_p, parsed));
     }
 
     return LY_SUCCESS;
@@ -1345,24 +1503,22 @@ lyb_parse_node_list(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, const s
         struct lyd_node **first_p, struct ly_set *parsed)
 {
     LY_ERR rc;
-    uint8_t peek;
     struct lyd_node *node = NULL;
     struct lyd_meta *meta = NULL;
-    uint32_t flags = 0;
+    uint32_t flags = 0, metadata_count;
     ly_bool log_node = 0;
 
     while (1) {
-        /* peek for the end instance flag (special metadata count) */
-        peek = 0;
-        lyb_peek(&peek, LYB_METADATA_END_BITS, lybctx->parse_ctx);
-        if (peek == LYB_METADATA_END) {
-            /* all the instances parsed, read the end flag */
-            lyb_read(&peek, LYB_METADATA_END_BITS, lybctx->parse_ctx);
+        /* read metadata count to check for end of instances */
+        metadata_count = 0;
+        lyb_read_count(&metadata_count, lybctx->parse_ctx);
+        if (metadata_count == LYB_METADATA_END_COUNT) {
+            /* all the instances parsed */
             break;
         }
 
         /* read necessary basic data */
-        rc = lyb_parse_node_header(lybctx, snode, &flags, &meta);
+        rc = lyb_parse_node_header(lybctx, snode, metadata_count, &flags, &meta);
         LY_CHECK_GOTO(rc, error);
 
         /* create list node */
@@ -1422,7 +1578,6 @@ lyb_parse_node(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, struct lyd_n
     const struct lysc_node *snode;
     const struct lys_module *mod;
     enum lylyb_node_type lyb_type = 0;
-    char *mod_name = NULL, mod_rev[LY_REV_SIZE];
 
     /* read node type */
     lyb_read(&lyb_type, LYB_NODE_TYPE_BITS, lybctx->parse_ctx);
@@ -1437,7 +1592,11 @@ lyb_parse_node(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, struct lyd_n
         goto cleanup;
     case LYB_NODE_TOP:
         /* top-level, read module */
-        LY_CHECK_GOTO(rc = lyb_parse_module_idx(lybctx->parse_ctx, &mod), cleanup);
+        if (lybctx->parse_ctx->shrink) {
+            LY_CHECK_GOTO(rc = lyb_parse_module_idx(lybctx, &mod), cleanup);
+        } else {
+            LY_CHECK_GOTO(rc = lyb_parse_module(lybctx, &mod), cleanup);
+        }
 
         /* read hash, find the schema node starting from mod */
         LY_CHECK_GOTO(rc = lyb_parse_schema_hash(lybctx, NULL, mod, &snode), cleanup);
@@ -1447,11 +1606,11 @@ lyb_parse_node(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, struct lyd_n
         LY_CHECK_GOTO(rc = lyb_parse_schema_hash(lybctx, lyd_parser_node_schema(parent), NULL, &snode), cleanup);
         break;
     case LYB_NODE_EXT:
-        /* ext, read module name */
-        LY_CHECK_GOTO(rc = lyb_parse_module(lybctx->parse_ctx, &mod_name, mod_rev), cleanup);
+        /* ext, read module name, always unshrinked format */
+        LY_CHECK_GOTO(rc = lyb_parse_module(lybctx, &mod), cleanup);
 
         /* read schema node name, find the nested ext schema node */
-        LY_CHECK_GOTO(rc = lyb_parse_schema_nested_ext(lybctx, parent, mod_name, &snode), cleanup);
+        LY_CHECK_GOTO(rc = lyb_parse_schema_nested_ext(lybctx, parent, mod->name, &snode), cleanup);
         break;
     }
 
@@ -1466,12 +1625,11 @@ lyb_parse_node(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, struct lyd_n
     } else if (snode->nodetype & LYD_NODE_INNER) {
         rc = lyb_parse_node_inner(lybctx, parent, snode, first_p, parsed);
     } else {
-        rc = lyb_parse_node_leaf(lybctx, parent, snode, first_p, parsed);
+        rc = lyb_parse_node_leaf(lybctx, parent, snode, UINT32_MAX, first_p, parsed);
     }
     LY_CHECK_GOTO(rc, cleanup);
 
 cleanup:
-    free(mod_name);
     return rc;
 }
 
@@ -1504,6 +1662,45 @@ lyb_parse_siblings(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, ly_bool
     return LY_SUCCESS;
 }
 
+/**
+ * @brief Parse context hash and optionally compare it to the current context hash.
+ *
+ * @note Hashes are compared only if the shrinked format is used.
+ *
+ * @param[in] lybctx LYB context.
+ * @return LY_ERR value.
+ */
+static LY_ERR
+lyb_parse_context_hash(struct lyd_lyb_ctx *lybctx)
+{
+    uint32_t data_hash, cur_hash;
+    struct lylyb_parse_ctx *pctx = lybctx->parse_ctx;
+
+    /* context hash */
+    data_hash = 0;
+    lyb_read((uint8_t *)&data_hash, LYB_HEADER_CTX_HASH_BITS, pctx);
+
+    /* correct byte order */
+    data_hash = le32toh(data_hash);
+
+    if (!data_hash) {
+        /* fine for no data */
+        pctx->empty_hash = 1;
+    } else if (lybctx->parse_ctx->shrink) {
+        /* only compare hashes in shrinked format, they must exactly match, because we will be accessing modules by indexes.
+         * truncate context hash to the same bit size */
+        cur_hash = lyb_truncate_hash_nonzero(ly_ctx_get_modules_hash(pctx->ctx), LYB_HEADER_CTX_HASH_BITS);
+
+        if (data_hash != cur_hash) {
+            LOGERR(pctx->ctx, LY_EINVAL, "Different current LYB context modules hash compared to the one stored in the "
+                    "LYB data (0x%" PRIx32 " != 0x%" PRIx32 ").", data_hash, cur_hash);
+            return LY_EINVAL;
+        }
+    }
+
+    return LY_SUCCESS;
+}
+
 /**
  * @brief Parse LYB header.
  *
@@ -1513,10 +1710,12 @@ lyb_parse_siblings(struct lyd_lyb_ctx *lybctx, struct lyd_node *parent, ly_bool
 static LY_ERR
 lyb_parse_header(struct lyd_lyb_ctx *lybctx)
 {
-    uint8_t byte;
-    uint32_t data_hash, cur_hash;
+    uint8_t byte, is_shrink, remaining_bit_count;
     struct lylyb_parse_ctx *pctx = lybctx->parse_ctx;
 
+    /* assume shrinked format until we read the flag in the header, so we can parse the header bit by bit */
+    pctx->shrink = 1;
+
     /* version */
     byte = 0;
     lyb_read(&byte, LYB_HEADER_VERSION_BITS, pctx);
@@ -1538,30 +1737,19 @@ lyb_parse_header(struct lyd_lyb_ctx *lybctx)
     /* shrink */
     byte = 0;
     lyb_read(&byte, LYB_HEADER_SHRINK_FLAG_BITS, pctx);
+    is_shrink = byte;
+
+    /* read and check remaining reserved bits */
+    remaining_bit_count = 8 - (LYB_HEADER_VERSION_BITS + LYB_HEADER_HASH_ALG_BITS + LYB_HEADER_SHRINK_FLAG_BITS);
+    byte = 0;
+    lyb_read(&byte, remaining_bit_count, pctx);
     if (byte) {
-        pctx->shrink = 1;
+        LOGERR(pctx->ctx, LY_EINVAL, "Invalid reserved bits in LYB header.");
+        return LY_EINVAL;
     }
 
-    /* context hash */
-    data_hash = 0;
-    lyb_read((uint8_t *)&data_hash, LYB_HEADER_CTX_HASH_BITS, pctx);
-
-    /* correct byte order */
-    data_hash = le32toh(data_hash);
-
-    if (!data_hash) {
-        /* fine for no data */
-        pctx->empty_hash = 1;
-    } else if (!(lybctx->parse_opts & LYD_PARSE_LYB_SKIP_CTX_CHECK)) {
-        /* truncate context hash to the same bit size */
-        cur_hash = lyb_truncate_hash_nonzero(ly_ctx_get_modules_hash(pctx->ctx), LYB_HEADER_CTX_HASH_BITS);
-
-        if (data_hash != cur_hash) {
-            LOGERR(pctx->ctx, LY_EINVAL, "Different current LYB context modules hash compared to the one stored in the "
-                    "LYB data (0x%" PRIx32 " != 0x%" PRIx32 ").", data_hash, cur_hash);
-            return LY_EINVAL;
-        }
-    }
+    /* set the real shrink value */
+    pctx->shrink = is_shrink;
 
     return LY_SUCCESS;
 }
@@ -1608,6 +1796,10 @@ lyd_parse_lyb(const struct ly_ctx *ctx, const struct lysc_ext_instance *ext, str
     rc = lyb_parse_header(lybctx);
     LY_CHECK_GOTO(rc, cleanup);
 
+    /* read and check context hash */
+    rc = lyb_parse_context_hash(lybctx);
+    LY_CHECK_GOTO(rc, cleanup);
+
     /* read sibling(s) */
     rc = lyb_parse_siblings(lybctx, parent, 1, first_p, parsed);
     LY_CHECK_GOTO(rc, cleanup);
