@@ -153,7 +153,7 @@ lys_compile_ext_instance_stmt(struct lysc_ctx *ctx, void **parsed_p, struct lysc
     case LY_STMT_LEAF_LIST:
     case LY_STMT_LIST:
     case LY_STMT_USES: {
-        const uint16_t flags;
+        uint16_t flags;
         struct lysp_node *pnodes, *pnode;
         struct lysc_node *node;
 
@@ -231,7 +231,7 @@ lys_compile_ext_instance_stmt(struct lysc_ctx *ctx, void **parsed_p, struct lysc
         break;
     }
     case LY_STMT_WHEN: {
-        const uint16_t flags;
+        uint16_t flags;
         const struct lysp_when *when = *parsed_p;
 
         /* read compiled status */
@@ -288,7 +288,7 @@ lys_compile_ext_instance_stmt(struct lysc_ctx *ctx, void **parsed_p, struct lysc
         break;
 
     case LY_STMT_TYPE: {
-        const uint16_t flags;
+        uint16_t flags;
         const char *units;
         const struct lysp_type *ptype = *parsed_p;
 
@@ -787,7 +787,9 @@ lyplg_ext_set_parent_ctx(struct ly_ctx *ctx, const struct ly_ctx *parent_ctx)
              * contexts, there is no ext callback for freeing the compiled extension data with the contexts) */
             ly_ctx_destroy(ctx);
         } else {
-            /* remove its shared and private data */
+            /* remove its shared and private data, this is an exception as we need to free compiled patterns
+             * manually, since we are not destroying the whole context, we will just be using the parent's ctx data instead */
+            ly_ctx_pattern_ht_erase(ctx);
             ly_ctx_data_del(ctx);
         }
     } else if (!parent_ctx) {
