@@ -332,6 +332,12 @@ lyb_write(const void *buf, uint64_t count_bits, struct lylyb_print_ctx *lybctx)
         return LY_SUCCESS;
     }
 
+    if (!lybctx->shrink) {
+        /* just write full bytes */
+        count_bytes = LYPLG_BITS2BYTES(count_bits);
+        return ly_write_(lybctx->out, buf, count_bytes);
+    }
+
     if (lybctx->buf_bits + count_bits < 8) {
         /* only buffering additional bits, append them (from left) */
         lybctx->buf |= ((*(uint8_t *)buf) << lybctx->buf_bits) & lyb_left_bit_mask(8 - lybctx->buf_bits);
@@ -408,10 +414,25 @@ lyb_write_flush(struct lylyb_print_ctx *lybctx)
 static LY_ERR
 lyb_write_count(uint32_t count, struct lylyb_print_ctx *lybctx)
 {
-    uint8_t prefix_b, num_b;
+    uint8_t prefix_b, num_b, byte_len;
+    uint16_t buf16;
     uint32_t buf;
 
-    /* prepare prefix in buf (in reverse, read bit-by-bit), set prefix length and number length in bits */
+    /* --- no shrink mode --- */
+    if (!lybctx->shrink) {
+        if (count > UINT16_MAX) {
+            LOGERR(lybctx->ctx, LY_EINT, "Cannot print count %" PRIu32 ", largest supported number is %" PRIu16 ".", count, UINT16_MAX);
+            return LY_EINT;
+        }
+
+        /* always write the count on 2 bytes */
+        byte_len = 2;
+        buf16 = htole16(count);
+        return lyb_write(&buf16, byte_len * 8, lybctx);
+    }
+
+    /* --- shrink mode ---
+     * prepare prefix in buf (in reverse, read bit-by-bit), set prefix length and number length in bits */
     if (count == 0) {
         /* 0 */
         buf = 0;
@@ -466,10 +487,19 @@ lyb_write_count(uint32_t count, struct lylyb_print_ctx *lybctx)
 static LY_ERR
 lyb_write_size(uint32_t size, struct lylyb_print_ctx *lybctx)
 {
-    uint8_t prefix_b, num_b;
+    uint8_t prefix_b, num_b, byte_len;
     uint32_t buf;
 
-    /* prepare prefix in buf (in reverse, read bit-by-bit), set prefix length and number length in bits */
+    /* --- no shrink mode --- */
+    if (!lybctx->shrink) {
+        /* always write the size on 4 bytes */
+        byte_len = 4;
+        buf = htole32(size);
+        return lyb_write(&buf, byte_len * 8, lybctx);
+    }
+
+    /* --- shrink mode ---
+     * prepare prefix in buf (in reverse, read bit-by-bit), set prefix length and number length in bits */
     if (size < 16) {
         /* prefix 0, encoded on 5 b */
         buf = 0x0;
@@ -565,7 +595,7 @@ cleanup:
 }
 
 /**
- * @brief Print YANG module info as only the module index in the context.
+ * @brief Print YANG module info as name, revision and enabled features.
  *
  * @param[in] mod Module to print.
  * @param[in] lybctx Printer LYB context.
@@ -577,6 +607,7 @@ lyb_print_module(const struct lys_module *mod, struct lylyb_print_ctx *lybctx)
     LY_ERR rc = LY_SUCCESS;
     uint16_t revision;
     int r;
+    LY_ARRAY_COUNT_TYPE i;
 
     /* module name length and module name */
     LY_CHECK_GOTO(rc = lyb_write_string(mod->name, 0, lybctx), cleanup);
@@ -604,10 +635,40 @@ lyb_print_module(const struct lys_module *mod, struct lylyb_print_ctx *lybctx)
     }
     LY_CHECK_GOTO(rc = lyb_write(&revision, 2 * 8, lybctx), cleanup);
 
+    /* feature count */
+    LY_CHECK_GOTO(rc = lyb_write_count(LY_ARRAY_COUNT(mod->compiled->features), lybctx), cleanup);
+
+    /* features */
+    LY_ARRAY_FOR(mod->compiled->features, i) {
+        LY_CHECK_GOTO(rc = lyb_write_string(mod->compiled->features[i], 0, lybctx), cleanup);
+    }
+
 cleanup:
     return rc;
 }
 
+/**
+ * @brief Print context hash.
+ *
+ * @param[in] lybctx Printer LYB context.
+ * @return LY_ERR value.
+ */
+static LY_ERR
+lyb_print_context_hash(struct lylyb_print_ctx *lybctx)
+{
+    uint32_t hash = 0;
+
+    /* context hash (is truncated), if not printing empty data */
+    if (lybctx->ctx) {
+        hash = lyb_truncate_hash_nonzero(ly_ctx_get_modules_hash(lybctx->ctx), LYB_HEADER_CTX_HASH_BITS);
+
+        /* correct byte order */
+        hash = htole32(hash);
+    }
+
+    return lyb_write(&hash, LYB_HEADER_CTX_HASH_BITS, lybctx);
+}
+
 /**
  * @brief Print LYB header.
  *
@@ -617,8 +678,14 @@ cleanup:
 static LY_ERR
 lyb_print_header(struct lylyb_print_ctx *lybctx)
 {
-    uint8_t byte = 0;
-    uint32_t hash;
+    uint8_t byte = 0, remaining_bit_count;
+    ly_bool is_shrink;
+
+    /* save the shrink mode */
+    is_shrink = lybctx->shrink;
+
+    /* temporarily set shrink to 1 to allow writing the header bit by bit */
+    lybctx->shrink = 1;
 
     /* version */
     byte = LYB_HEADER_VERSION_NUM;
@@ -629,19 +696,16 @@ lyb_print_header(struct lylyb_print_ctx *lybctx)
     LY_CHECK_RET(lyb_write(&byte, LYB_HEADER_HASH_ALG_BITS, lybctx));
 
     /* shrink */
-    byte = (lybctx->shrink ? 1 : 0);
+    byte = (is_shrink ? 1 : 0);
     LY_CHECK_RET(lyb_write(&byte, LYB_HEADER_SHRINK_FLAG_BITS, lybctx));
 
-    /* context hash (is truncated), if not printing empty data */
-    if (lybctx->ctx) {
-        hash = lyb_truncate_hash_nonzero(ly_ctx_get_modules_hash(lybctx->ctx), LYB_HEADER_CTX_HASH_BITS);
+    /* fill remaining reserved bits with 0 */
+    remaining_bit_count = 8 - (LYB_HEADER_VERSION_BITS + LYB_HEADER_HASH_ALG_BITS + LYB_HEADER_SHRINK_FLAG_BITS);
+    byte = 0;
+    LY_CHECK_RET(lyb_write(&byte, remaining_bit_count, lybctx));
 
-        /* correct byte order */
-        hash = htole32(hash);
-    } else {
-        hash = 0;
-    }
-    LY_CHECK_RET(lyb_write(&hash, LYB_HEADER_CTX_HASH_BITS, lybctx));
+    /* restore the shrink mode */
+    lybctx->shrink = is_shrink;
 
     return LY_SUCCESS;
 }
@@ -801,9 +865,9 @@ lyb_print_metadata(const struct lyd_node *node, struct lyd_lyb_ctx *lybctx)
         if (!lyd_metadata_should_print(iter)) {
             continue;
         }
-        if (count == LYB_METADATA_END - 1) {
+        if (count == LYB_METADATA_END_COUNT - 1) {
             LOGERR(lybctx->print_ctx->ctx, LY_EINT, "Maximum supported number of data node metadata is %" PRIu8 ".",
-                    LYB_METADATA_END - 1);
+                    LYB_METADATA_END_COUNT - 1);
             return LY_EINT;
         }
         ++count;
@@ -814,7 +878,11 @@ lyb_print_metadata(const struct lyd_node *node, struct lyd_lyb_ctx *lybctx)
 
     if (df_mod) {
         /* write the "default" metadata */
-        LY_CHECK_RET(lyb_print_module_idx(df_mod, lybctx->print_ctx));
+        if (lybctx->print_ctx->shrink) {
+            LY_CHECK_RET(lyb_print_module_idx(df_mod, lybctx->print_ctx));
+        } else {
+            LY_CHECK_RET(lyb_print_module(df_mod, lybctx->print_ctx));
+        }
         LY_CHECK_RET(lyb_write_string("default", 0, lybctx->print_ctx));
         LY_CHECK_RET(lyb_write_string("true", 0, lybctx->print_ctx));
     }
@@ -826,7 +894,11 @@ lyb_print_metadata(const struct lyd_node *node, struct lyd_lyb_ctx *lybctx)
         }
 
         /* module */
-        LY_CHECK_RET(lyb_print_module_idx(iter->annotation->module, lybctx->print_ctx));
+        if (lybctx->print_ctx->shrink) {
+            LY_CHECK_RET(lyb_print_module_idx(iter->annotation->module, lybctx->print_ctx));
+        } else {
+            LY_CHECK_RET(lyb_print_module(iter->annotation->module, lybctx->print_ctx));
+        }
 
         /* annotation name with length */
         LY_CHECK_RET(lyb_write_string(iter->name, 0, lybctx->print_ctx));
@@ -982,12 +1054,17 @@ lyb_print_schema_hash(struct lysc_node *schema, struct ly_ht **sibling_ht, struc
 static LY_ERR
 lyb_print_node_header(const struct lyd_node *node, struct lyd_lyb_ctx *lybctx)
 {
+    uint32_t buf;
+
     /* write any metadata */
     LY_CHECK_RET(lyb_print_metadata(node, lybctx));
 
     if (!lybctx->print_ctx->shrink) {
+        /* correct byte order */
+        buf = htole32(node->flags);
+
         /* write node flags, fixed bits */
-        LY_CHECK_RET(lyb_write(&node->flags, LYB_DATA_NODE_FLAG_BITS, lybctx->print_ctx));
+        LY_CHECK_RET(lyb_write(&buf, LYB_DATA_NODE_FLAG_BITS, lybctx->print_ctx));
     }
 
     return LY_SUCCESS;
@@ -1251,9 +1328,16 @@ lyb_print_node(const struct lyd_node **printed_node, struct ly_ht **sibling_ht,
 
     /* write module info first */
     if (node->flags & LYD_EXT) {
+        /* always write full module info for exts */
         LY_CHECK_RET(lyb_print_module(node->schema->module, lybctx->print_ctx));
     } else if (node->schema && !lysc_data_parent(node->schema)) {
-        LY_CHECK_RET(lyb_print_module_idx(node->schema->module, lybctx->print_ctx));
+        if (lybctx->print_ctx->shrink) {
+            /* write just module index in shrink mode */
+            LY_CHECK_RET(lyb_print_module_idx(node->schema->module, lybctx->print_ctx));
+        } else {
+            /* write full module info in non-shrink mode */
+            LY_CHECK_RET(lyb_print_module(node->schema->module, lybctx->print_ctx));
+        }
     }
 
     if (node->flags & LYD_EXT) {
@@ -1360,6 +1444,9 @@ lyb_print_data(struct ly_out *out, const struct lyd_node *root, uint32_t options
     /* LYB header */
     LY_CHECK_GOTO(rc = lyb_print_header(lybctx->print_ctx), cleanup);
 
+    /* context hash */
+    LY_CHECK_GOTO(rc = lyb_print_context_hash(lybctx->print_ctx), cleanup);
+
     /* all the top-level siblings, recursively */
     LY_CHECK_GOTO(rc = lyb_print_siblings(root, 1, lybctx), cleanup);
 
