@@ -26,6 +26,7 @@
 #include "dict.h"
 #include "log.h"
 #include "ly_common.h"
+#include "plugins_internal.h"
 #include "schema_compile.h"
 #include "schema_compile_node.h"
 #include "schema_features.h"
@@ -53,8 +54,8 @@ lysc_nodeid_free(const struct ly_ctx *ctx, struct lysc_nodeid *nodeid)
     }
 
     for (i = 0; i < nodeid->count; ++i) {
-        lydict_remove(ctx, nodeid->prefix[i]);
-        lydict_remove(ctx, nodeid->name[i]);
+        lysdict_remove(ctx, nodeid->prefix[i]);
+        lysdict_remove(ctx, nodeid->name[i]);
     }
     free(nodeid->prefix);
     free(nodeid->name);
@@ -124,7 +125,7 @@ relative_path:
         ptr = ly_strnchr(name, ':', len);
         if (ptr) {
             /* store prefix */
-            rc = lydict_insert(ctx, name, ptr - name, &(*nodeid)->prefix[(*nodeid)->count - 1]);
+            rc = lysdict_insert(ctx, name, ptr - name, &(*nodeid)->prefix[(*nodeid)->count - 1]);
             LY_CHECK_GOTO(rc, cleanup);
 
             /* move name */
@@ -133,14 +134,14 @@ relative_path:
         }
 
         /* store name */
-        rc = lydict_insert(ctx, name, len, &(*nodeid)->name[(*nodeid)->count - 1]);
+        rc = lysdict_insert(ctx, name, len, &(*nodeid)->name[(*nodeid)->count - 1]);
         LY_CHECK_GOTO(rc, cleanup);
 
         ++i;
     }
 
 cleanup:
-    lyxp_expr_free(ctx, exp);
+    lyxp_expr_free(exp);
     if (rc) {
         lysc_nodeid_free(ctx, *nodeid);
         *nodeid = NULL;
@@ -279,7 +280,7 @@ lys_nodeid_mod_check(struct lysc_ctx *ctx, const char *str, ly_bool abs, struct
     }
 
 cleanup:
-    lyxp_expr_free(ctx->ctx, e);
+    lyxp_expr_free(e);
     lysc_nodeid_free(ctx->ctx, ni);
     return ret;
 }
@@ -472,24 +473,24 @@ lysp_ext_dup(const struct ly_ctx *ctx, const struct lysp_module *pmod, void *par
     LY_ERR ret = LY_SUCCESS;
     struct ly_set pmods = {0};
     struct lysp_ctx pctx = {.parsed_mods = &pmods};
+    struct lyplg_ext *ext_plg;
 
     DUP_STRING_GOTO(ctx, orig_ext->name, ext->name, ret, cleanup);
     DUP_STRING_GOTO(ctx, orig_ext->argument, ext->argument, ret, cleanup);
     ext->format = orig_ext->format;
     LY_CHECK_GOTO(ret = ly_dup_prefix_data(ctx, orig_ext->format, orig_ext->prefix_data, &ext->prefix_data), cleanup);
-    ext->def = orig_ext->def;
+    ext->plugin_ref = orig_ext->plugin_ref;
 
     ext->parent = parent;
     ext->parent_stmt = parent_stmt;
     ext->parent_stmt_index = orig_ext->parent_stmt_index;
     ext->flags = orig_ext->flags;
-    ext->record = orig_ext->record;
 
     LY_CHECK_GOTO(ret = lysp_ext_children_dup(ctx, orig_ext->child, &ext->child), cleanup);
-    if (ext->record && ext->record->plugin.parse) {
+    if (ext->plugin_ref && (ext_plg = LYSC_GET_EXT_PLG(ext->plugin_ref))->parse) {
         /* parse again */
         LY_CHECK_GOTO(ret = ly_set_add(&pmods, pmod, 1, NULL), cleanup);
-        LY_CHECK_GOTO(ret = ext->record->plugin.parse(&pctx, ext), cleanup);
+        LY_CHECK_GOTO(ret = ext_plg->parse(&pctx, ext), cleanup);
     }
 
 cleanup:
@@ -915,7 +916,7 @@ lys_apply_refine(struct lysc_ctx *ctx, struct lysp_refine *rfn, const struct lys
         case LYS_LEAF:
             AMEND_CHECK_CARDINALITY(rfn->dflts, 1, "refine", "default");
 
-            lydict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->dflt.str);
+            lysdict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->dflt.str);
             LY_CHECK_GOTO(ret = lysp_qname_dup(ctx->ctx, &rfn->dflts[0], &((struct lysp_node_leaf *)target)->dflt), cleanup);
             break;
         case LYS_LEAFLIST:
@@ -936,7 +937,7 @@ lys_apply_refine(struct lysc_ctx *ctx, struct lysp_refine *rfn, const struct lys
         case LYS_CHOICE:
             AMEND_CHECK_CARDINALITY(rfn->dflts, 1, "refine", "default");
 
-            lydict_remove(ctx->ctx, ((struct lysp_node_choice *)target)->dflt.str);
+            lysdict_remove(ctx->ctx, ((struct lysp_node_choice *)target)->dflt.str);
             LY_CHECK_GOTO(ret = lysp_qname_dup(ctx->ctx, &rfn->dflts[0], &((struct lysp_node_choice *)target)->dflt), cleanup);
             break;
         default:
@@ -946,13 +947,13 @@ lys_apply_refine(struct lysc_ctx *ctx, struct lysp_refine *rfn, const struct lys
 
     /* description */
     if (rfn->dsc) {
-        lydict_remove(ctx->ctx, target->dsc);
+        lysdict_remove(ctx->ctx, target->dsc);
         DUP_STRING_GOTO(ctx->ctx, rfn->dsc, target->dsc, ret, cleanup);
     }
 
     /* reference */
     if (rfn->ref) {
-        lydict_remove(ctx->ctx, target->ref);
+        lysdict_remove(ctx->ctx, target->ref);
         DUP_STRING_GOTO(ctx->ctx, rfn->ref, target->ref, ret, cleanup);
     }
 
@@ -990,7 +991,7 @@ lys_apply_refine(struct lysc_ctx *ctx, struct lysp_refine *rfn, const struct lys
             AMEND_WRONG_NODETYPE("refine", "replace", "presence");
         }
 
-        lydict_remove(ctx->ctx, ((struct lysp_node_container *)target)->presence);
+        lysdict_remove(ctx->ctx, ((struct lysp_node_container *)target)->presence);
         DUP_STRING_GOTO(ctx->ctx, rfn->presence, ((struct lysp_node_container *)target)->presence, ret, cleanup);
     }
 
@@ -1344,7 +1345,7 @@ lys_apply_deviate_delete(struct lysc_ctx *ctx, struct lysp_deviate_del *d, struc
         }
 
         DEV_CHECK_PRESENCE_VALUE(struct lysp_node_leaf *, units, "deleting", "units", d->units);
-        lydict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->units);
+        lysdict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->units);
         ((struct lysp_node_leaf *)target)->units = NULL;
     }
 
@@ -1355,7 +1356,7 @@ lys_apply_deviate_delete(struct lysc_ctx *ctx, struct lysp_deviate_del *d, struc
             AMEND_WRONG_NODETYPE("deviation", "delete", "must");
         }
 
-        DEV_DEL_ARRAY(musts, *musts, .arg.str, .arg.str, lysp_restr_free, &ctx->free_ctx, "must");
+        DEV_DEL_ARRAY(musts, *musts, .arg.str, .arg.str, lysp_restr_free, ctx->ctx, "must");
     }
 
     /* *unique-stmt */
@@ -1375,7 +1376,7 @@ lys_apply_deviate_delete(struct lysc_ctx *ctx, struct lysp_deviate_del *d, struc
             AMEND_CHECK_CARDINALITY(d->dflts, 1, "deviation", "default");
             DEV_CHECK_PRESENCE_VALUE(struct lysp_node_leaf *, dflt.str, "deleting", "default", d->dflts[0].str);
 
-            lydict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->dflt.str);
+            lysdict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->dflt.str);
             ((struct lysp_node_leaf *)target)->dflt.str = NULL;
             break;
         case LYS_LEAFLIST:
@@ -1386,7 +1387,7 @@ lys_apply_deviate_delete(struct lysc_ctx *ctx, struct lysp_deviate_del *d, struc
             AMEND_CHECK_CARDINALITY(d->dflts, 1, "deviation", "default");
             DEV_CHECK_PRESENCE_VALUE(struct lysp_node_choice *, dflt.str, "deleting", "default", d->dflts[0].str);
 
-            lydict_remove(ctx->ctx, ((struct lysp_node_choice *)target)->dflt.str);
+            lysdict_remove(ctx->ctx, ((struct lysp_node_choice *)target)->dflt.str);
             ((struct lysp_node_choice *)target)->dflt.str = NULL;
             break;
         default:
@@ -1429,7 +1430,7 @@ lys_apply_deviate_replace(struct lysc_ctx *ctx, struct lysp_deviate_rpl *d, stru
             AMEND_WRONG_NODETYPE("deviation", "replace", "type");
         }
 
-        lysp_type_free(&ctx->free_ctx, &((struct lysp_node_leaf *)target)->type);
+        lysp_type_free(ctx->ctx, &((struct lysp_node_leaf *)target)->type);
         lysp_type_dup(ctx->ctx, ctx->pmod, d->type, &((struct lysp_node_leaf *)target)->type);
     }
 
@@ -1444,7 +1445,7 @@ lys_apply_deviate_replace(struct lysc_ctx *ctx, struct lysp_deviate_rpl *d, stru
         }
 
         DEV_CHECK_PRESENCE(struct lysp_node_leaf *, units, "replacing", "units", d->units);
-        lydict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->units);
+        lysdict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->units);
         DUP_STRING_GOTO(ctx->ctx, d->units, ((struct lysp_node_leaf *)target)->units, ret, cleanup);
     }
 
@@ -1454,13 +1455,13 @@ lys_apply_deviate_replace(struct lysc_ctx *ctx, struct lysp_deviate_rpl *d, stru
         case LYS_LEAF:
             DEV_CHECK_PRESENCE(struct lysp_node_leaf *, dflt.str, "replacing", "default", d->dflt.str);
 
-            lydict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->dflt.str);
+            lysdict_remove(ctx->ctx, ((struct lysp_node_leaf *)target)->dflt.str);
             LY_CHECK_GOTO(ret = lysp_qname_dup(ctx->ctx, &d->dflt, &((struct lysp_node_leaf *)target)->dflt), cleanup);
             break;
         case LYS_CHOICE:
             DEV_CHECK_PRESENCE(struct lysp_node_choice *, dflt.str, "replacing", "default", d->dflt.str);
 
-            lydict_remove(ctx->ctx, ((struct lysp_node_choice *)target)->dflt.str);
+            lysdict_remove(ctx->ctx, ((struct lysp_node_choice *)target)->dflt.str);
             LY_CHECK_GOTO(ret = lysp_qname_dup(ctx->ctx, &d->dflt, &((struct lysp_node_choice *)target)->dflt), cleanup);
             break;
         default:
@@ -1625,33 +1626,6 @@ lysp_schema_nodeid_match_node(const struct lysc_node **node, const struct lys_mo
     return 1;
 }
 
-/**
- * @brief Check whether a compiled ext instance matches a single schema nodeid name test.
- *
- * @param[in,out] ext Compiled ext instance to consider. On a match it is zeroed to not match again.
- * @param[in] mod Expected module.
- * @param[in] name Expected name, in the dictionary.
- * @return Whether it is a match or not.
- */
-static ly_bool
-lysp_schema_nodeid_match_ext(const struct lysc_ext_instance **ext, const struct lys_module *mod, const char *name_dict)
-{
-    /* compare with the module */
-    if ((*ext)->module != mod) {
-        return 0;
-    }
-
-    /* compare names (argument) */
-    if ((*ext)->argument != name_dict) {
-        return 0;
-    }
-
-    /* zero */
-    *ext = NULL;
-
-    return 1;
-}
-
 /**
  * @brief Check whether a node matches specific schema nodeid.
  *
@@ -1704,7 +1678,7 @@ lysp_schema_nodeid_match(const struct lysc_nodeid *nodeid, const struct lysp_mod
     while (i) {
         --i;
 
-        if (!parent && !pnode_ext) {
+        if (!parent) {
             /* no more parents but path continues */
             return 0;
         }
@@ -1713,24 +1687,17 @@ lysp_schema_nodeid_match(const struct lysc_nodeid *nodeid, const struct lysp_mod
         mod = lys_schema_node_get_module(nodeid_pmod->mod->ctx, nodeid->prefix[i], nodeid_pmod);
         assert(mod);
 
-        if (parent) {
-            /* compare with the parent */
-            if (!lysp_schema_nodeid_match_node(&parent, mod, nodeid->name[i])) {
-                return 0;
-            }
-        } else {
-            /* compare with the ext instance */
-            if (!lysp_schema_nodeid_match_ext(&pnode_ext, mod, nodeid->name[i])) {
-                return 0;
-            }
+        /* compare with the parent */
+        if (!lysp_schema_nodeid_match_node(&parent, mod, nodeid->name[i])) {
+            return 0;
         }
     }
 
     if (ctx_node && (ctx_node != parent)) {
         /* descendant path has not finished in the context node */
         return 0;
-    } else if (!ctx_node && (parent || pnode_ext)) {
-        /* some parent/extension was not matched */
+    } else if (!ctx_node && parent) {
+        /* some parent was not matched */
         return 0;
     }
 
@@ -1814,7 +1781,7 @@ lysp_dev_node_free(struct lysc_ctx *cctx, struct lysp_node *dev_pnode)
     case LYS_INPUT:
     case LYS_OUTPUT:
         ((struct lysp_node_action_inout *)dev_pnode)->child = NULL;
-        lysp_node_free(&cctx->free_ctx, dev_pnode);
+        lysp_node_free(cctx->ctx, dev_pnode);
         free(dev_pnode);
         return;
     default:
@@ -1822,7 +1789,7 @@ lysp_dev_node_free(struct lysc_ctx *cctx, struct lysp_node *dev_pnode)
         return;
     }
 
-    lysp_node_free(&cctx->free_ctx, dev_pnode);
+    lysp_node_free(cctx->ctx, dev_pnode);
 }
 
 LY_ERR
@@ -2550,7 +2517,7 @@ lys_precompile_augments_deviations(struct lys_module *mod, struct lys_glob_unres
 
         if (!m->implemented) {
             /* implement the target module */
-            imp_f = (mod->ctx->flags & LY_CTX_ENABLE_IMP_FEATURES) ? all_f : NULL;
+            imp_f = (mod->ctx->opts & LY_CTX_ENABLE_IMP_FEATURES) ? all_f : NULL;
             r = lys_implement(m, imp_f, unres);
             if (r == LY_ERECOMPILE) {
                 /* implement all the modules right away to save possible later recompilation */
@@ -2581,8 +2548,8 @@ lys_precompile_augments_deviations_revert(struct ly_ctx *ctx, const struct lys_m
     LY_ARRAY_COUNT_TYPE u, count;
     struct lys_module *m;
 
-    for (i = 0; i < ctx->list.count; ++i) {
-        m = ctx->list.objs[i];
+    for (i = 0; i < ctx->modules.count; ++i) {
+        m = ctx->modules.objs[i];
 
         if (m->augmented_by) {
             count = LY_ARRAY_COUNT(m->augmented_by);
