@@ -105,7 +105,7 @@ lyd_dup_inst_get(const struct lyd_node *first_inst, struct ly_ht **dup_inst_ht)
 }
 
 LY_ERR
-lyd_dup_inst_next(struct lyd_node **inst, const struct lyd_node *siblings, struct ly_ht **dup_inst_ht)
+lyd_dup_inst_next(struct lyd_node **inst, struct ly_ht **dup_inst_ht)
 {
     struct lyd_dup_inst *dup_inst;
 
@@ -117,11 +117,11 @@ lyd_dup_inst_next(struct lyd_node **inst, const struct lyd_node *siblings, struc
     /* there can be more exact same instances (even if not allowed in invalid data) and we must make sure we do not
      * match a single node more times */
     dup_inst = lyd_dup_inst_get(*inst, dup_inst_ht);
-    LY_CHECK_ERR_RET(!dup_inst, LOGMEM(LYD_CTX(siblings)), LY_EMEM);
+    LY_CHECK_ERR_RET(!dup_inst, LOGMEM(LYD_CTX(*inst)), LY_EMEM);
 
     if (!dup_inst->used) {
-        /* we did not cache these instances yet, do so */
-        lyd_find_sibling_dup_inst_set(siblings, *inst, &dup_inst->set);
+        /* we did not cache these instances yet, do so (use the same inst in case it is from a mount-point) */
+        lyd_find_sibling_dup_inst_set(*inst, *inst, &dup_inst->set);
         assert(dup_inst->set->count && (dup_inst->set->dnodes[0] == *inst));
     }
 
@@ -1840,79 +1840,59 @@ ly_time_ts2str(const struct timespec *ts, char **str)
     return ly_time_time2str(ts->tv_sec, ts->tv_nsec ? frac_buf : NULL, str);
 }
 
-LY_ERR
-ly_pattern_code_match(const pcre2_code *pcode, const char *str, size_t str_len, struct ly_err_item **err)
-{
-    int r, match_opts;
-    pcre2_match_data *match_data = NULL;
-
-    /* match_data needs to be allocated each time because of possible multi-threaded evaluation */
-    match_data = pcre2_match_data_create_from_pattern(pcode, NULL);
-    if (!match_data) {
-        return ly_err_new(err, LY_EMEM, 0, NULL, NULL, LY_EMEM_MSG);
-    }
-
-    match_opts = PCRE2_ANCHORED;
-#ifdef PCRE2_ENDANCHORED
-    /* PCRE2_ENDANCHORED was added in PCRE2 version 10.30 */
-    match_opts |= PCRE2_ENDANCHORED;
-#endif
-    r = pcre2_match(pcode, (PCRE2_SPTR)str, str_len, 0, match_opts, match_data, NULL);
-    pcre2_match_data_free(match_data);
-
-    if ((r != PCRE2_ERROR_NOMATCH) && (r < 0)) {
-        PCRE2_UCHAR pcre2_errmsg[LY_PCRE2_MSG_LIMIT] = {0};
-
-        pcre2_get_error_message(r, pcre2_errmsg, LY_PCRE2_MSG_LIMIT);
-        return ly_err_new(err, LY_ESYS, 0, NULL, NULL, "%s", (const char *)pcre2_errmsg);
-    }
-
-    return (r == PCRE2_ERROR_NOMATCH) ? LY_ENOT : LY_SUCCESS;
-}
-
 LIBYANG_API_DEF LY_ERR
-ly_pattern_match(const struct ly_ctx *ctx, const char *pattern, const char *string, uint32_t str_len, pcre2_code **pcode)
+ly_pattern_match(const struct ly_ctx *ctx, const char *pattern, const char *string, uint32_t str_len, void **pat_comp)
 {
     LY_ERR r;
-    pcre2_code *code;
     struct ly_err_item *err = NULL;
 
-    LY_CHECK_ARG_RET(ctx, pattern || (pcode && *pcode), string, LY_EINVAL);
+    LY_CHECK_ARG_RET(ctx, pattern || (pat_comp && *pat_comp), string, LY_EINVAL);
 
-    if (pcode && *pcode) {
-        /* use precompiled pattern */
-        code = *pcode;
-    } else {
-        /* compile the pattern */
-        LY_CHECK_RET(lys_compile_type_pattern_check(ctx, pattern, &code));
+    /* compile, if needed and requested */
+    if (pat_comp && !*pat_comp) {
+        r = ly_pat_compile(pattern, 0, pat_comp, &err);
+        if (r) {
+            ly_err_print(ctx, err);
+            ly_err_free(err);
+            return r;
+        }
     }
 
     /* match */
     if (!str_len) {
         str_len = strlen(string);
     }
-    r = ly_pattern_code_match(code, string, str_len, &err);
+    r = ly_pat_match((pat_comp && *pat_comp) ? *pat_comp : NULL, pattern, 0, string, str_len, &err);
     if (r && (r != LY_ENOT)) {
         ly_err_print(ctx, err);
+    }
+    ly_err_free(err);
+
+    return r;
+}
+
+LIBYANG_API_DEF LY_ERR
+ly_pattern_compile(const struct ly_ctx *ctx, const char *pattern, void **pat_comp)
+{
+    LY_ERR r;
+    struct ly_err_item *err = NULL;
+
+    LY_CHECK_ARG_RET(ctx, pattern, pat_comp, LY_EINVAL);
+
+    *pat_comp = NULL;
+
+    r = ly_pat_compile(pattern, 0, pat_comp, &err);
+    if (r) {
+        ly_err_print(ctx, err);
         ly_err_free(err);
         return r;
     }
 
-    if (pcode && !*pcode) {
-        *pcode = code;
-    } else if (!pcode || !*pcode) {
-        pcre2_code_free(code);
-    }
-    return r;
+    return LY_SUCCESS;
 }
 
-LIBYANG_API_DEF LY_ERR
-ly_pattern_compile(const struct ly_ctx *ctx, const char *pattern, pcre2_code **pcode)
+LIBYANG_API_DEF void
+ly_pattern_free(void *pat_comp)
 {
-    LY_CHECK_ARG_RET(ctx, pattern, pcode, LY_EINVAL);
-
-    *pcode = NULL;
-
-    /* compile the pattern */
-    return lys_compile_type_pattern_check(ctx, pattern, pcode);
+    ly_pat_free(pat_comp, 0);
 }
