@@ -47,11 +47,10 @@ struct lyd_dup_inst {
  * (that should not be considered next time) instance.
  *
  * @param[in,out] inst Found instance, is updated so that the same instance is not returned twice.
- * @param[in] siblings Siblings where @p inst was found.
  * @param[in] dup_inst_ht Duplicate instance cache hash table.
  * @return LY_ERR value.
  */
-LY_ERR lyd_dup_inst_next(struct lyd_node **inst, const struct lyd_node *siblings, struct ly_ht **dup_inst_ht);
+LY_ERR lyd_dup_inst_next(struct lyd_node **inst, struct ly_ht **dup_inst_ht);
 
 /**
  * @brief Free duplicate instance cache.
@@ -831,17 +830,4 @@ LY_ERR lyd_unlink_leafref_node(const struct lyd_node_term *node, const struct ly
  */
 void lyd_unlink_ignore_lyds(struct lyd_node **first_sibling, struct lyd_node *node);
 
-/**
- * @brief Check a string matches a compiled pattern.
- *
- * @param[in] pcode Compiled pattern.
- * @param[in] str String to match.
- * @param[in] str_len Length of @p str.
- * @param[out] err Error, if generated.
- * @return LY_SUCCESS on a match;
- * @return LY_ENOT if the string does not match;
- * @return LY_ERR on error.
- */
-LY_ERR ly_pattern_code_match(const pcre2_code *pcode, const char *str, size_t str_len, struct ly_err_item **err);
-
 #endif /* LY_TREE_DATA_INTERNAL_H_ */
