@@ -1948,6 +1948,8 @@ lysp_add_internal_yang(struct lysp_ctx *pctx, struct lysp_module *mod)
     struct lysp_ext_instance *extp, *prev_exts = mod->exts;
     struct lysp_stmt *stmt;
     struct lysp_tpdf *tpdf;
+    struct lysp_node_leaf *leaf;
+    struct lysp_import *imp;
     uint32_t idx;
 
     /* add new typedef */
@@ -1989,6 +1991,20 @@ lysp_add_internal_yang(struct lysp_ctx *pctx, struct lysp_module *mod)
         pctx->ext_inst.objs[idx] = mod->exts;
     }
 
+    /* add a date-and-time leaf so that such values can be validated (there is a compiled type) */
+    LY_LIST_NEW_RET(mod->mod->ctx, &mod->data, leaf, next, LY_EMEM);
+    leaf->nodetype = LYS_LEAF;
+    LY_CHECK_RET(lysdict_insert(mod->mod->ctx, "date-and-time", 0, &leaf->name));
+    leaf->flags = LYS_INTERNAL;
+    LY_CHECK_RET(lysdict_insert(mod->mod->ctx, "yang_:date-and-time", 0, &leaf->type.name));
+    leaf->type.pmod = mod;
+
+    /* create new imports for the used prefixes */
+    LY_ARRAY_NEW_RET(mod->mod->ctx, mod->imports, imp, LY_EMEM);
+    LY_CHECK_RET(lysdict_insert(mod->mod->ctx, "ietf-yang-types", 0, &imp->name));
+    LY_CHECK_RET(lysdict_insert(mod->mod->ctx, "yang_", 0, &imp->prefix));
+    imp->flags = LYS_INTERNAL;
+
     return LY_SUCCESS;
 }
 
@@ -2349,13 +2365,28 @@ lys_search_localfile_file_type(const struct dirent *file, const char *wd, struct
 {
     LY_ERR rc = LY_SUCCESS;
     char *str = NULL;
-    ly_bool is_dir = 0, is_reg = 0;
+    ly_bool is_dir = 0, is_reg = 0, need_stat = 1;
     struct stat st;
 
     *skip = 0;
 
-    if ((file->d_type == DT_UNKNOWN) || (file->d_type == DT_LNK)) {
-        /* FS does not support this field or its a symbolic link, need to call stat */
+#ifdef HAVE_DIRENT_D_TYPE
+    if (file->d_type == DT_DIR) {
+        /* dirent - dir */
+        is_dir = 1;
+        need_stat = 0;
+    } else if (file->d_type == DT_REG) {
+        /* dirent - file */
+        is_reg = 1;
+        need_stat = 0;
+    } else if ((file->d_type != DT_UNKNOWN) && (file->d_type != DT_LNK)) {
+        /* it is a known type, but not dir or regular file, so trust d_type and just skip it */
+        need_stat = 0;
+    }
+#endif // HAVE_DIRENT_D_TYPE
+
+    if (need_stat) {
+        /* need to use stat to determine the file type */
         if (asprintf(&str, "%s/%s", wd, file->d_name) == -1) {
             LOGMEM(NULL);
             rc = LY_EMEM;
@@ -2371,12 +2402,6 @@ lys_search_localfile_file_type(const struct dirent *file, const char *wd, struct
             /* stat - file */
             is_reg = 1;
         }
-    } else if (file->d_type == DT_DIR) {
-        /* dirent - dir */
-        is_dir = 1;
-    } else if (file->d_type == DT_REG) {
-        /* dirent - file */
-        is_reg = 1;
     }
 
     if (is_dir && (dirs->count || !implicit_cwd)) {
