@@ -181,14 +181,6 @@ struct lysp_yin_ctx {
     struct lyxml_ctx *xmlctx;        /**< context for xml parser */
 };
 
-/**
- * @brief Internal pattern hash table record.
- */
-struct ly_pattern_ht_rec {
-    const char *pattern;            /**< Pattern string, used both as key to hash and value to search for. */
-    pcre2_code *pcode;              /**< Compiled PCRE2 pattern code. */
-};
-
 /**
  * @brief Check that @p c is valid UTF8 code point for YANG string.
  *
