@@ -508,7 +508,7 @@ test_parse_inline(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -550,7 +550,11 @@ test_parse_inline(void **state)
             "      <type xmlns:ianaift=\"urn:ietf:params:xml:ns:yang:iana-if-type\">ianaift:ethernetCsmacd</type>\n"
             "      <oper-status>not-present</oper-status>\n"
             "      <statistics>\n"
+#ifdef ENABLE_DATE_AND_TIME_TYPE_COMPAT
             "        <discontinuity-time>2022-01-01T10:00:00-00:00</discontinuity-time>\n"
+#else
+            "        <discontinuity-time>2022-01-01T10:00:00Z</discontinuity-time>\n"
+#endif
             "      </statistics>\n"
             "    </interface>\n"
             "  </interfaces-state>\n"
@@ -578,7 +582,11 @@ test_parse_inline(void **state)
             "          \"type\": \"iana-if-type:ethernetCsmacd\",\n"
             "          \"oper-status\": \"not-present\",\n"
             "          \"statistics\": {\n"
+#ifdef ENABLE_DATE_AND_TIME_TYPE_COMPAT
             "            \"discontinuity-time\": \"2022-01-01T10:00:00-00:00\"\n"
+#else
+            "            \"discontinuity-time\": \"2022-01-01T10:00:00Z\"\n"
+#endif
             "          }\n"
             "        }\n"
             "      ]\n"
@@ -628,7 +636,7 @@ test_parse_inline(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -715,7 +723,7 @@ test_parse_shared(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -758,7 +766,11 @@ test_parse_shared(void **state)
             "      <type xmlns:ianaift=\"urn:ietf:params:xml:ns:yang:iana-if-type\">ianaift:ethernetCsmacd</type>\n"
             "      <oper-status>not-present</oper-status>\n"
             "      <statistics>\n"
+#ifdef ENABLE_DATE_AND_TIME_TYPE_COMPAT
             "        <discontinuity-time>2022-01-01T10:00:00-00:00</discontinuity-time>\n"
+#else
+            "        <discontinuity-time>2022-01-01T10:00:00Z</discontinuity-time>\n"
+#endif
             "      </statistics>\n"
             "    </interface>\n"
             "  </interfaces-state>\n"
@@ -785,7 +797,11 @@ test_parse_shared(void **state)
             "          \"type\": \"iana-if-type:ethernetCsmacd\",\n"
             "          \"oper-status\": \"not-present\",\n"
             "          \"statistics\": {\n"
+#ifdef ENABLE_DATE_AND_TIME_TYPE_COMPAT
             "            \"discontinuity-time\": \"2022-01-01T10:00:00-00:00\"\n"
+#else
+            "            \"discontinuity-time\": \"2022-01-01T10:00:00Z\"\n"
+#endif
             "          }\n"
             "        }\n"
             "      ]\n"
@@ -835,7 +851,7 @@ test_parse_shared(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -921,7 +937,7 @@ test_parse_shared(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -972,7 +988,7 @@ test_parse_shared(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -1012,7 +1028,11 @@ test_parse_shared(void **state)
             "      <type xmlns:ianaift=\"urn:ietf:params:xml:ns:yang:iana-if-type\">ianaift:ethernetCsmacd</type>\n"
             "      <oper-status>not-present</oper-status>\n"
             "      <statistics>\n"
+#ifdef ENABLE_DATE_AND_TIME_TYPE_COMPAT
             "        <discontinuity-time>2022-01-01T10:00:00-00:00</discontinuity-time>\n"
+#else
+            "        <discontinuity-time>2022-01-01T10:00:00Z</discontinuity-time>\n"
+#endif
             "      </statistics>\n"
             "    </interface>\n"
             "  </interfaces-state>\n"
@@ -1030,7 +1050,11 @@ test_parse_shared(void **state)
             "      <type xmlns:ianaift=\"urn:ietf:params:xml:ns:yang:iana-if-type\">ianaift:fddi</type>\n"
             "      <oper-status>down</oper-status>\n"
             "      <statistics>\n"
+#ifdef ENABLE_DATE_AND_TIME_TYPE_COMPAT
             "        <discontinuity-time>2020-01-01T10:00:00-00:00</discontinuity-time>\n"
+#else
+            "        <discontinuity-time>2022-01-01T10:00:00Z</discontinuity-time>\n"
+#endif
             "      </statistics>\n"
             "    </interface>\n"
             "  </interfaces-state>\n"
@@ -1057,7 +1081,11 @@ test_parse_shared(void **state)
             "          \"type\": \"iana-if-type:ethernetCsmacd\",\n"
             "          \"oper-status\": \"not-present\",\n"
             "          \"statistics\": {\n"
+#ifdef ENABLE_DATE_AND_TIME_TYPE_COMPAT
             "            \"discontinuity-time\": \"2022-01-01T10:00:00-00:00\"\n"
+#else
+            "            \"discontinuity-time\": \"2022-01-01T10:00:00Z\"\n"
+#endif
             "          }\n"
             "        }\n"
             "      ]\n"
@@ -1079,7 +1107,11 @@ test_parse_shared(void **state)
             "          \"type\": \"iana-if-type:fddi\",\n"
             "          \"oper-status\": \"down\",\n"
             "          \"statistics\": {\n"
+#ifdef ENABLE_DATE_AND_TIME_TYPE_COMPAT
             "            \"discontinuity-time\": \"2020-01-01T10:00:00-00:00\"\n"
+#else
+            "            \"discontinuity-time\": \"2020-01-01T10:00:00Z\"\n"
+#endif
             "          }\n"
             "        }\n"
             "      ]\n"
@@ -1143,7 +1175,7 @@ test_parse_shared_parent_ref(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -1306,7 +1338,7 @@ test_dup_shared(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -1455,7 +1487,7 @@ test_parse_config(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -1563,7 +1595,7 @@ test_parse_config(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -1665,7 +1697,7 @@ test_new(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -1708,7 +1740,11 @@ test_new(void **state)
             "      <type xmlns:ianaift=\"urn:ietf:params:xml:ns:yang:iana-if-type\">ianaift:ethernetCsmacd</type>\n"
             "      <oper-status>not-present</oper-status>\n"
             "      <statistics>\n"
+#ifdef ENABLE_DATE_AND_TIME_TYPE_COMPAT
             "        <discontinuity-time>2022-01-01T10:00:00-00:00</discontinuity-time>\n"
+#else
+            "        <discontinuity-time>2022-01-01T10:00:00Z</discontinuity-time>\n"
+#endif
             "      </statistics>\n"
             "    </interface>\n"
             "  </interfaces-state>\n"
@@ -1801,7 +1837,7 @@ test_lys_getnext(void **state)
             "    </module>"
             "    <import-only-module>"
             "      <name>ietf-yang-types</name>"
-            "      <revision>2013-07-15</revision>"
+            "      <revision>2025-12-22</revision>"
             "      <namespace>urn:ietf:params:xml:ns:yang:ietf-yang-types</namespace>"
             "    </import-only-module>"
             "  </module-set>"
@@ -1833,6 +1869,10 @@ test_lys_getnext(void **state)
 
     node = lys_getnext(NULL, parent, NULL, LYS_GETNEXT_WITHSCHEMAMOUNT);
     assert_non_null(node);
+    assert_string_equal(node->name, "date-and-time");
+
+    node = lys_getnext(node, parent, NULL, LYS_GETNEXT_WITHSCHEMAMOUNT);
+    assert_non_null(node);
     assert_string_equal(node->name, "schema-mounts");
 
     node = lys_getnext(node, parent, NULL, LYS_GETNEXT_WITHSCHEMAMOUNT);
@@ -1931,7 +1971,7 @@ test_xpath(void **state)
             "</root>\n";
     CHECK_PARSE_LYD_PARAM(xml, LYD_XML, LYD_PARSE_STRICT, LYD_VALIDATE_PRESENT, LY_EVALID, data);
     CHECK_LOG_CTX("Invalid instance-identifier \"/m:root/m:l3\" value - semantic error: Not found node \"l3\" in path.",
-            "/mount:root/l2", 4);
+            "/sm:root/mount:root/l2", 4);
 
     /* valid */
     xml =
