@@ -238,9 +238,17 @@ help(int shortout)
 static void
 libyang_verbclb(LY_LOG_LEVEL level, const char *msg, const char *data_path, const char *schema_path, uint64_t line)
 {
+    LY_LOG_LEVEL sr_ll;
     const char *levstr;
     char *full_msg = NULL, *aux;
 
+    sr_ll = ly_log_level(0);
+    ly_log_level(sr_ll);
+
+    if (level > sr_ll) {
+        return;
+    }
+
     switch (level) {
     case LY_LLERR:
         levstr = "err :";
@@ -515,7 +523,7 @@ process_args(int argc, char *argv[], struct yl_opt *yo, struct ly_ctx **ctx)
     yo->line_length = 0;
 
     opterr = 0;
-    while ((opt = getopt_long(argc, argv, "hvVQf:I:p:DF:iP:qs:neEA:t:d:lL:o:O:R:myY:XJx:G:k:", options, &opt_index)) != -1) {
+    while ((opt = getopt_long(argc, argv, "hvVQf:I:p:DF:iP:qs:neE:At:d:lL:o:O:R:myY:XJx:G:k:", options, &opt_index)) != -1) {
         switch (opt) {
         case 'h': /* --help */
             help(0);
